#include "stdafx.h"
#include "CppUnitTest.h"
#include <windows.h>
#include <string>
#include <algorithm>
#include "sorts.h"
using namespace std;

using namespace Microsoft::VisualStudio::CppUnitTestFramework;

//
//void someSort(int* thingToSort, int n)
//{
//	int* oneBasedArray = thingToSort - 1;
//
//	Assert::isTrue(oneBasedArray[1] == thingToSort[0]);
//
//}

namespace UnitTest1
{		
	template <typename Type>
	void initArray(Type* original, Type* duplicate, 
		int mask,
		int length, bool isAlreadySorted, int seed = 2423413)
	{
		srand(seed);
		for (int i = 0; i < length; i++)
		{
			// rand() produces 15 bits of rand data
			// (rand() << 15) produces:
			//    xxaaaaaaaaaaaaaaa000000000000000
			// shift the original rand number 15 bits to the left
			// Use or operator to place another random number in the right-most bits
			//   xxaaaaaaaaaaaaaaabbbbbbbbbbbbbbb
			// Move the whole thing one more bit to the left
			//   xaaaaaaaaaaaaaaabbbbbbbbbbbbbbb0
			// Use or operator to add the final (right-most bit)
			// Use the mask to take the desired number of right-most bits
			// as the value

			original[i] = ((((rand() << 15) | rand()) << 1) |
				(rand() & 1)) & mask;

			// if you want already sorted data
			if (isAlreadySorted)
			{
				std::sort(original, original + length);
			}

			duplicate[i] = original[i];
		}
	}



	TEST_CLASS(UnitTest1)
	{
	private:
		static const int MAX_ARRAY_SIZE = 10000; // later 10000000
		static int* originalData;
		static int* duplicateData;

		static unsigned long testSort(const string& description,
			void(*sortFunction)(int* startPtr, int* endPtr),
			int mask, int length)
		{
			// Variables to store start and end time
			LARGE_INTEGER startTime;
			LARGE_INTEGER endTime;

			// Create random data
			initArray(originalData, duplicateData, mask, length, false);

			// Time the sort
			QueryPerformanceCounter(&startTime);
			(*sortFunction)(originalData, originalData + length - 1);
			QueryPerformanceCounter(&endTime);

			unsigned long sortTime = endTime.QuadPart 
				- startTime.QuadPart;

			// Write the sort time
			stringstream timeString;
			timeString << sortTime;
			Logger::WriteMessage(description.c_str());
			Logger::WriteMessage(timeString.str().c_str());

			// Check to be sure that the sort really works.
			std::sort(duplicateData, duplicateData + length);

			for (int i = 0; i < length; i++)
			{
				Assert::AreEqual(duplicateData[i], originalData[i]);
			}

			// Return the sort time
			return sortTime;
		}

		static unsigned long* runAllTests(const string& description,
			void(*sortFunction)(int* startPtr, int* endPtr), int& nTestsRun)
		{
			const int N_TESTS = 3;
			nTestsRun = N_TESTS;
			unsigned long* results = new unsigned long[nTestsRun];

			// 
			int length = 10;
			int mask = 1023; // 0x1ff
			results[0] = testSort(description, sortFunction, mask, length);

			length = 100;
			results[1] = testSort(description, sortFunction, mask, length);

			length = 1000;
			results[2] = testSort(description, sortFunction, mask, length);

			return results;
		}

	public:

		TEST_CLASS_INITIALIZE(makeArrays)
		{
			originalData = new int[MAX_ARRAY_SIZE];
			duplicateData = new int[MAX_ARRAY_SIZE];
		}

		TEST_CLASS_CLEANUP(deleteArrays)
		{
			delete[] originalData;
			delete[] duplicateData;
		}
		
		TEST_METHOD(librarySortTest)
		{
			// TODO: Your test code here
			int nTests;
			unsigned long* result = runAllTests("librarySort", 
				::libSort<int>, nTests);
		}

		TEST_METHOD(bubbleSort)
		{
			// TODO: Your test code here
			int nTests;
			unsigned long* result = runAllTests("bubbleSort", 
				::bubbleSort<int>, nTests);
		}



	};

	int* UnitTest1::originalData;
	int* UnitTest1::duplicateData;
}